// program rec2tif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "tiffio.h"

#define INTEL

double	*data;		/* raw data by CBP */
double	delta;		/* pixel size */

short	KDIMU;
short	NDIMU;		/* size of CT image */
short	NANG;		/* number of porjection */
short	LTYPE;		/* filter type */
short	nlayer;		/* calculation layer */
double	AXISU;		/* rotation center*/

// total absorption
double	TAM, TASD;

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}

/*----------------------------------------------------------------------*/
int read_image(x_size, y_size)
short	x_size, y_size;
{
	FILE	*fi;
	long	i,j;
	char	fname[15];

//open input file
	sprintf(fname,"r%04d.rec",nlayer);
	if((fi = fopen(fname,"rb")) == NULL){
		printf("can not open rec.dat for input\n");
		return(-10);
	}

	data = (double *) malloc(x_size*y_size*sizeof(double));
	if ((j = fread(data, sizeof(double), x_size*y_size, fi)) != x_size*y_size){
		fclose(fi);
		fprintf(stderr, " Error reading rec.dat at %d\n", j);
		return(1);
	}

	for(i=0;i<x_size*y_size;++i)
		*(data+i) = *(data+i)*10000./delta;	/* unit change  um -> cm */

//	for(i=0;i<y_size;++i)
//		for(j=0;j<x_size;++j)
//		printf("%d\t%d\t%lf\n",j,i,*(data+i*x_size+j));

	fclose(fi);
	
	if( unlink(fname) == -1 ){
		fprintf(stderr,"%s 폜ł܂",fname);
		return(1);
	}
	else{
		return(0);
	}
}


/*----------------------------------------------------------------------*/

void	StoreTiffFile(path,Nx,Ny,BPS,cell,bell,desc)
char	*path,*desc;
short	Nx,Ny,BPS;
unsigned short *cell;
unsigned char *bell;
{
	TIFF	*image;
	long	i, j, k, l;

	image = TIFFOpen(path, "w");

	TIFFSetField(image, TIFFTAG_IMAGEWIDTH, Nx);
	TIFFSetField(image, TIFFTAG_IMAGELENGTH, Ny);
	TIFFSetField(image, TIFFTAG_BITSPERSAMPLE, BPS);
	TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
//	TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_LZW);
	TIFFSetField(image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
	TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL, 1);
	TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, 1);
//	TIFFSetField(image, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);// ȂĂ悢
//	TIFFSetField(image, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB); // for intel
//	TIFFSetField(image, TIFFTAG_FILLORDER, FILLORDER_LSB2MSB); // for motorola
//	TIFFSetField(image, TIFFTAG_STRIPBYTECOUNTS,h.width*sizeof(short));
//	TIFFSetField(image, TIFFTAG_DATATYPE, TIFF_SHORT);  // ȂĂ悢
	TIFFSetField(image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(image, TIFFTAG_IMAGEDESCRIPTION, desc);
	TIFFSetField(image, TIFFTAG_ARTIST, "SP-uCT, rec2tif");

	if(BPS==16){
		for(i = 0; i < Ny; ++i)
			TIFFWriteRawStrip(image, i, cell+i*Nx, Nx*sizeof(short));
	}
	else if (BPS==8){
		for(i = 0; i < Ny; ++i){
			TIFFWriteRawStrip(image, i, bell+i*Nx, Nx*sizeof(char));
		}
	}

	TIFFClose(image);
}
/*----------------------------------------------------------------------*/

int	main(argc,argv)
int	argc;
char	**argv;
{
	char	*comm = NULL;
	char	fout[15];

	int		i, j;
	short	Nx, Ny;

	int		y,x;
	long	val;

	double	to;
	short	nbit;

	FILE	*f;
	char	lne[20], *ss;
	double	data_max, data_min;
	float	base;
	long	lup;
	double	div;
	unsigned short	*cell;
	unsigned char	*bell;

	long	s_dum;
	long	l_dum;
	float	f_dum;
	double	d_dum;

	nbit  = 16;			// default 16bit
	if (argc>1){
		s_dum=atoi(argv[1]);
		nbit =s_dum;
	}
	delta = 1.0;		// default pixel size (micron)
	if (argc>2){
		d_dum=atof(argv[2]);
		delta=d_dum;
	}
	div   = 100000.;	// conversion const. from LAC to pixel value
	if (argc>3){
		d_dum=atof(argv[3]);
		div  =d_dum;
	}
	base   = 0.0;	    // base value for conversion const.
	if (argc>4){
		f_dum=(float)atof(argv[4]);
		base=f_dum;
	}

	if((nbit!=8)&&(nbit!=16)) Error("8 or 16bit");
	
	if((f = fopen("rec.tmp","r")) == NULL){
		printf("can not open 'rec.tmp' for input\n");
		return(-10);
	}
	lne[0]='\0';ss=fgets(lne, 100, f); nlayer=atoi(lne);
	lne[0]='\0';ss=fgets(lne, 100, f); KDIMU=atoi(lne);
	lne[0]='\0';ss=fgets(lne, 100, f); NANG=atoi(lne);
	lne[0]='\0';ss=fgets(lne, 100, f); NDIMU=atoi(lne);
	lne[0]='\0';ss=fgets(lne, 100, f); AXISU=atof(lne);
	lne[0]='\0';ss=fgets(lne, 100, f); LTYPE=atoi(lne);
	lne[0]='\0';ss=fgets(lne, 100, f); TAM=atof(lne);
	lne[0]='\0';ss=fgets(lne, 100, f); TASD=atof(lne);
	
	fclose(f);
//	if( unlink( "rec.tmp" ) == -1 ){
//		perror( "'rec.dat' 폜ł܂" );
//		return(1);
//	}

	//printf("%d\t%d\t%d\t%d\t%lf\t%d\n",nlayer,KDIMU,NANG,NDIMU,AXISU,LTYPE);

	Nx = NDIMU;
	Ny = NDIMU;

	if ((i = read_image(Nx,Ny)) != 0){
		printf("something wrong -- return value is %d", i);
		return(-11);
	}
//printf("%d\t%d\t%lf\n", Nx,Ny, *(data+301*301-1)*1000000.);

// search max and min
	data_max =-32000.;
	data_min = 32000.;
	for (i=0; i<Nx*Ny;i++){
		if(data_max<*(data+i)) data_max=*(data+i);
		if(data_min>*(data+i)) data_min=*(data+i);
	}
	to = data_max;

// calc conversion const.
	if(argc<4){
		div=((pow(2,nbit))/(data_max-data_min));
		base = (float)data_min;
	}

	printf("Image size:\t%d x %d\n", Nx, Ny);
	printf("MAX and min:\t%f and %f\n", (float)data_max, (float)data_min);
	printf("CT image to:\t%d bits TIFF.\n", nbit);
	printf("Conv. const.:\t%lf\t%f\n",div, base);
	printf("pixel size:\t%lf\n",delta);

// initialize cell and bell
	cell = (unsigned short *)malloc(Nx*Ny*sizeof(short));
	bell = (unsigned char *)malloc(Nx*Ny*sizeof(char));

	j = 0;
	lup = (long)pow(2,nbit)-1;
	for (y=0; y<Ny; y++){
		for (x=0; x<Nx; x++){
			val = (long)(div*(*(data+j)-base));
			if(val < 1) val = 0;
			if(val > lup) val = lup;
			*(cell+j)=(unsigned short)val;
			*(bell+j)=(unsigned char)val;
			j=j+1;
		}
	}

    if ((comm=(char *)malloc(150))==NULL)
		Error("comment memory allocation error.");
	sprintf(comm,"%d\t%lf\t%lf\t%f\t%lf\t%d\t%lf\t%lf",NANG,delta,div,base,AXISU,LTYPE,TAM,TASD);
	//sprintf(desc, "%lf\t%lf\n%d\t%lf\n%lf\t%lf\n%lf\t%lf", delta, r0, M, theta0, f1, f2, f0, df);

	sprintf(fout,"rec%04d.tif",nlayer);
	StoreTiffFile(fout,Nx,Ny,nbit,cell,bell,comm);
	printf("Stored to:\t%s\n",fout);
	
	free(cell);
	free(bell);

	return 0;
}
